//
//  LXTinyLoadingView.m
//  LXTinyLoadingViewDemo
//
//  Created by poPLax on 2013-07-2.
//  Copyright (c) 2013 Poplax Studio. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "LXTinyLoadingView.h"

#define LX_BUNDLE_PATH                                 @"LoadingView.bundle/"
#define LX_LOADING_VIEW_PREFIX                         @"lx_loading_"

#define LX_TINY_LOADING_VIEW_DEFAULT_SIZE              20.f
#define LX_TINY_LOADING_VIEW_CHANGE_ANGLE              0.12f
#define LX_TINY_LOADING_VIEW_CHANGE_ANGLE_PER_TIME_MAX 6.29f

@interface LXTinyLoadingView ()
{
    BOOL _isHiddenAnimating;
    BOOL _animating;
    CGFloat _angle;
    UIImage *_image;
    UIImageView *_loadingImageView;
}

@end

@implementation LXTinyLoadingView

- (id)initWithFrame:(CGRect)frame
{
    //    CGRect fitFrame = { frame.origin, { LX_TINY_LOADING_VIEW_DEFAULT_SIZE, LX_TINY_LOADING_VIEW_DEFAULT_SIZE } };
    //    if (self = [super initWithFrame:fitFrame]) {
    if (self = [super initWithFrame:frame]) {
        [self setupView];
    }
    return self;
}

- (void)setImageName:(NSString *)suffixImageName
{
    if (suffixImageName) {
        _image = [UIImage imageNamed:[NSString stringWithFormat:@"%@%@%@", LX_BUNDLE_PATH, LX_LOADING_VIEW_PREFIX, suffixImageName]];
        if (_image) {
            _loadingImageView.image = _image;
            [_loadingImageView sizeToFit];
            _loadingImageView.center = CGPointMake(self.frame.size.width * 0.5f, self.frame.size.height * 0.5f);
        }
    }
}

#pragma mark -
#pragma mark Animated Action

- (void)startAnimating
{
    _animating = YES;
    self.hidden = NO;
}

- (void)stopAnimating
{
    _animating = NO;
    self.hidden = _hidesWhenStopped;

    if (_resetsWhenStopped) {
        _angle = 0;
        CGAffineTransform transform = CGAffineTransformMakeRotation(0);
        _loadingImageView.transform = transform;
    }
}

#pragma mark -
#pragma mark Private Method

- (void)setupView
{
    //    init Data
    [self setResetsWhenStopped:YES];
    [self setHidesWhenStopped:YES];
    [self setUseFadeEffect:YES];
    [self setBackgroundColor:[UIColor clearColor]];
    _isHiddenAnimating = NO;
    if (nil == _loadingImageView) {
        _loadingImageView = [[UIImageView alloc] init];
    }

    _angle = 0;
    NSTimer *timer = [NSTimer timerWithTimeInterval:0.02 target:self selector:@selector(handleTimer:) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:timer forMode:NSRunLoopCommonModes];

    [self addSubview:_loadingImageView];
}

- (void)handleTimer:(NSTimer *)timer
{
    if (_animating) {
        _angle += LX_TINY_LOADING_VIEW_CHANGE_ANGLE;
    }

    if (_angle > LX_TINY_LOADING_VIEW_CHANGE_ANGLE_PER_TIME_MAX) {
        _angle = 0;
    }

    CGAffineTransform transform = CGAffineTransformMakeRotation(_angle);
    _loadingImageView.transform = transform;
}

- (void)setHidden:(BOOL)hidden
{
    if (_useFadeEffect) {
        if (!_isHiddenAnimating && hidden != self.isHidden) {
            CGFloat from, to;
            if (hidden) {
                from = 1.0f;
                to = 0;
            } else {
                self.layer.hidden = NO;
                from = 0;
                to = 1.0f;
            }

            [CATransaction begin];
            _isHiddenAnimating = YES;
            CABasicAnimation *anim = [CABasicAnimation animationWithKeyPath:@"opacity"];
            [anim setDuration:.3f];
            [anim setFromValue:[NSNumber numberWithFloat:from]];
            [anim setToValue:[NSNumber numberWithFloat:to]];
            [CATransaction setCompletionBlock:^{
                _isHiddenAnimating = NO;
                [super setHidden:hidden];
            }];
            [self.layer addAnimation:anim forKey:@"animateOpacity"];
            [CATransaction commit];
        }
    } else {
        [super setHidden:hidden];
    }
}

@end
